#!/usr/bin/env python

# Copyright (c) 2022 Exograd SAS.
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
# SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
# IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import argparse
import logging
import pathlib
import sys

sys.path.insert(0, str(pathlib.Path(__file__).parents[1]))

import eventline

logging.basicConfig(level=logging.INFO)
eventline.client.log.setLevel(logging.DEBUG)

parser = argparse.ArgumentParser(description="Execute an Eventline command.")
parser.add_argument(
    "--project",
    dest="project",
    type=str,
    default="default",
    help="the name of the project",
)
parser.add_argument("name", type=str, help="the name of the command")
parser.add_argument(
    "parameter", nargs="*", type=str, help="a command parameter (name=value)"
)
args = parser.parse_args()

client = eventline.APIClient(project_name=args.project)

# Fetch the command to obtain its identifier and the list of definitions for
# its parameters.
command = client.get_resource_by_name("command", args.name)


# Parse parameters passed as command line arguments according to their
# definition.
parameters = {}
parameter_defs = command.spec["data"]["parameters"]

for parameter_string in args.parameter:
    name, value_string = parameter_string.split("=")
    parameter_def = None
    for def_ in parameter_defs:
        if def_["name"] == name:
            parameter_def = def_
            break
    if parameter_def is None:
        print(f"unknown parameter {name}", file=sys.stderr)
        exit(1)
    type_ = parameter_def["type"]
    value = None
    if type_ == "number":
        try:
            value = int(value_string)
        except:
            try:
                value = float(value_string)
            except:
                print(f"invalid number '{value_string}'", file=sys.stderr)
                exit(1)
    elif type_ == "string":
        value = value_string
    elif type_ == "boolean":
        value_string = value_string.lower()
        if value_string == "true":
            value = True
        elif value_string == "false":
            value = False
        else:
            print(f"invalid boolean '{value_string}'", file=sys.stderr)
            exit(1)
    parameters[name] = value

for parameter_def in parameter_defs:
    if "default" in parameter_def:
        continue
    name = parameter_def["name"]
    if name not in parameters:
        print(f"missing parameter {name}", file=sys.stderr)
        exit(1)

# Execute the command
input_ = eventline.CommandExecutionInput(parameters)

execution = client.execute_command(command.id_, input_)

print(f"command executed")

nb_pipelines = len(execution.pipeline_ids)
if nb_pipelines == 1:
    print(f"{nb_pipelines} pipeline created")
else:
    print(f"{nb_pipelines} pipelines created")

print(execution.id_)



# Copyright (c) 2022 Exograd SAS.
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
# SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
# IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import pytest

from eventline.client import ClientError, APIError, Client


def test_invalid_endpoint():
    with pytest.raises(ClientError):
        Client(endpoint="ftp://eventline.net")


def test_build_uri_default_endpoint():
    c = Client()
    assert c.build_uri("/") == f"{Client.default_endpoint}/"
    assert c.build_uri("/status") == f"{Client.default_endpoint}/status"


def test_build_uri_custom_endpoint():
    c = Client(endpoint="https://example.com/test")
    assert c.build_uri("/") == "https://example.com/test/"
    assert c.build_uri("/status") == "https://example.com/test/status"


def test_send_request_status():
    c = Client()
    res = c.send_request("GET", "/status")
    assert res.status == 200


def test_send_request_error():
    c = Client()
    with pytest.raises(APIError) as exinfo:
        c.send_request("GET", "/does_not_exist")
    ex = exinfo.value
    assert ex.status == 404
    assert ex.error_code == "route_not_found"



#!/usr/bin/env python

# Copyright (c) 2022 Exograd SAS.
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
# SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
# IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import argparse
import json
import logging
import pathlib
import sys

sys.path.insert(0, str(pathlib.Path(__file__).parents[1]))

import eventline

logging.basicConfig(level=logging.INFO)
eventline.client.log.setLevel(logging.DEBUG)

parser = argparse.ArgumentParser(description="Create a custom event.")
parser.add_argument(
    "--project",
    dest="project",
    type=str,
    default="default",
    help="the name of the project",
)
parser.add_argument("name", type=str, help="the name of the event")
parser.add_argument(
    "data",
    nargs="?",
    type=str,
    help='the JSON object representing event data ("-" to read stdin)',
)
args = parser.parse_args()

client = eventline.APIClient(project_name=args.project)

# Parse event data
data_string = args.data
if data_string == "-":
    data_string = sys.stdin.read()

data = json.loads(data_string)

# Create the event
new_event = eventline.NewEvent("custom", args.name, data)

events = client.create_event(new_event)

nb_events = len(events)
if nb_events == 1:
    print(f"{nb_events} event created:")
else:
    print(f"{nb_events} events created:")

for event in events:
    print(event.id_)



#!/usr/bin/env python

# Copyright (c) 2022 Exograd SAS.
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
# SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
# IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import argparse
import logging
import pathlib
import sys

sys.path.insert(0, str(pathlib.Path(__file__).parents[1]))

from tabulate import tabulate

import eventline
import eventline.client

logging.basicConfig(level=logging.INFO)
eventline.client.log.setLevel(logging.DEBUG)

parser = argparse.ArgumentParser(
    description="List all accounts in the Eventline organization."
)
parser.add_argument(
    "--project",
    dest="project",
    type=str,
    default="default",
    help="the name of the project",
)
parser.add_argument(
    "--command",
    dest="command",
    type=str,
    default=None,
    help="the name of a command",
)
args = parser.parse_args()

client = eventline.APIClient(project_name=args.project)

# Obtain the command id if there is one
command = None
command_id = None
if args.command is not None:
    command = client.get_resource_by_name("command", args.command)
    command_id = command.id_

# Fetch and print command executions
table = []

cursor = eventline.Cursor(size=20, order="desc")
while cursor is not None:
    page = client.get_command_executions(command_id=command_id, cursor=cursor)
    for execution in page.elements:
        table.append(
            [
                execution.execution_time,
                execution.command_id,
                execution.event_id,
                len(execution.pipeline_ids),
            ]
        )
    cursor = page.next

print(tabulate(table, ["Date", "Command id", "Event id", "Nb pipelines"]))



# Copyright (c) 2022 Exograd SAS.
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
# SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
# IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

from typing import Any, Dict

from eventline.api_object import ReadableAPIObject


class Pipeline(ReadableAPIObject):
    """A pipeline."""

    def __init__(self) -> None:
        super().__init__("pipeline")

    def _read(self, data: Dict[str, Any]) -> None:
        self.id_ = self._read_string(data, "id")
        self.name = self._read_string(data, "name")
        self.org_id = self._read_string(data, "org_id")
        self.project_id = self._read_string(data, "project_id")
        self.creation_time = self._read_datetime(data, "creation_time")
        self.pipeline_id = self._read_optional_string(data, "pipeline_id")
        self.trigger_id = self._read_optional_string(data, "trigger_id")
        self.command_id = self._read_optional_string(data, "command_id")
        self.event_id = self._read_optional_string(data, "event_id")
        self.concurrent = self._read_optional_boolean(data, "concurrent")
        self.status = self._read_string(data, "status")
        self.start_time = self._read_optional_datetime(data, "start_time")
        self.end_time = self._read_optional_datetime(data, "end_time")
